"use strict";

// code that gets run when the browser loads the TODO App

/**
 * Given a todo item, insert it into the todo list. If this is 
 * the first item, we will also create the skeleton for the todo list
 * table
 * @param {Todo item} item 
 */
function insertTodo(item) {
    // grab the root of the app
    const todo = document.querySelector("#todo");

    // check if the table exists
    if(!todo.querySelector("#todo-table")) {
        // table does not exist
        // create it!
        const parser = new DOMParser();
        const table = parser.parseFromString(`
            <table id="todo-table" class="table">
                <thead>
                    <tr>
                        <th scope="col">Description</th>
                        <th scope="col">Due Date</th>
                        <th scope="col">Priority</th>
                        <th scope="col"></th>
                    </tr>
                </thead>
                <tbody id="todo-body"></tbody>
            </table>
        `, "text/html");
        // insert this snippet into the DOM
        todo.appendChild(table.body.firstElementChild);
    } // if-todo-table

    // add the todo to the table
    const tableBody = todo.querySelector("#todo-body");
    // an alternate way of creating DOM nodes and inserting them
    const row = document.createElement("tr");
    row.innerHTML = `
        <td>${item.description}</td>
        <td class="todo-date">${item.due}</td>
        <td>${item.priority}</td>
        <td><a href="" class="remove"><i class="bi bi-x-lg"></i></a></td>
    `;
    tableBody.appendChild(row);

    row.querySelector(".remove").addEventListener("click", (evt) => {
        // this function will get called when the X button is clicked
        // prevent the default click behavior from happening
        evt.preventDefault();
        // remove the row from the DOM
        tableBody.removeChild(row);
    })

}


// put the main program (what runs when the page loads) into an event
// handler for the load event on the window
window.addEventListener("load", () => {
    // handler when the button is clicked
    document.querySelector("#add").addEventListener("click", (evt) => {
        evt.preventDefault();

        const form = document.querySelector("form");

        const desc = form.querySelector("#description");
        const due = form.querySelector("#due");
        const priority = form.querySelector("#priority");

        // check the validity of the date
        if (isNaN(new Date(due.value).valueOf())) {
            due.setCustomValidity("Must be a valid date");
        } else {
            // clear validation errors
            due.setCustomValidity("");
        }

        if (!form.checkValidity()) {
            // there is an error to deal with
            // stop the propagation of the click event so no
            // other click handlers are processed
            evt.stopPropagation();

            form.classList.add('was-validated');
            return;
        }

        // YAY! The form is valid :-)
        const todo = {
            description: desc.value,
            due: new Date(due.value),
            priority: priority.value
        };

        insertTodo(todo);
    });

    // create a handler to update the table every second
    setInterval(() => {
        document.querySelectorAll(".todo-date").forEach((item) => {
            const date = new Date(item.textContent);
            const curr = new Date();
            if (date < curr) {
                item.parentElement.classList.add("table-danger");
            }
        });
    }, 1000);
});

